/*
 * Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms
 * and conditions in the IBM International Program License Agreement.
 *
 *  Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package com.ibm.xtools.modeler.ui.pde.examples.diagram.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.uml2.Package;

import com.ibm.xtools.emf.msl.ResourceSetModifyOperation;
import com.ibm.xtools.modeler.UMLModeler;
import com.ibm.xtools.modeler.ui.pde.examples.diagram.DiagramPlugin;
import com.ibm.xtools.notation.Diagram;
import com.ibm.xtools.notation.View;
import com.ibm.xtools.presentation.services.layout.ILayoutNodesProvider;
import com.ibm.xtools.uml.diagram.IUMLDiagramHelper;
import com.ibm.xtools.umlnotation.UMLDiagramKind;

/**
 * For each selected packages, this action delegate creates a diagram that
 * contains the package contents.
 * 
 * @see IWorkbenchWindowActionDelegate
 */
public class CreatePackageDiagram
	implements IWorkbenchWindowActionDelegate {

	/**
	 * Title of diagram creation operation
	 */
	private static final String KEY_DIAGRAM_OPERATION_TITLE = "diagramOperationTitle"; //$NON-NLS-1$

	/**
	 * Prefix for created diagram name
	 */
	private static final String KEY_DIAGRAM_NAME_PREFIX = "diagramNamePrefix"; //$NON-NLS-1$

	/**
	 * THe UMLDiagramHelper
	 */
	private IUMLDiagramHelper helper = UMLModeler.getUMLDiagramHelper();

	/**
	 * Constructor
	 */
	public CreatePackageDiagram() {
		//No-op
	}

	/**
	 * Walk the selected objects and creates a new diagram for each visited
	 * packages
	 * 
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {

		/* Get selection */
		final List elements = UMLModeler.getUMLUIHelper().getSelectedElements();
		
		/* Perform remaining work within a ResourceSetWriteOperation */
		try {
			UMLModeler.getEditingDomain().run(
				new ResourceSetModifyOperation(DiagramPlugin
					.getResourceString(KEY_DIAGRAM_OPERATION_TITLE)) {

					protected void execute(IProgressMonitor monitor)
						throws InvocationTargetException, InterruptedException {

						/* create a diagram for each package */
						for (Iterator iter = elements.iterator(); iter
							.hasNext();) {
							Object object = iter.next();
							
							/* selection may have been from a Package View, get Package from view */
							if (object instanceof View) {
								object = ((View)object).getElement();
							}
							
							/* if element is indeed a package, then create a contents diagram */
							if (object instanceof Package) {
								createPackageContentsDiagram((Package) object);
							}
						}

					}
				}, new NullProgressMonitor());

		} catch (Exception e) {
			throw new RuntimeException(e.getCause());
		}
	}

	/**
	 * Creates a diagram in the specified package that will contain all of this
	 * diagram's elements that are representable in a diagram
	 * 
	 * @param aPackage
	 *            The package where the diagram is to be created and whose
	 *            content will be added to the diagram
	 */
	private void createPackageContentsDiagram(Package aPackage) {

		Diagram diagram = helper.createDiagram(aPackage,
			UMLDiagramKind.FREEFORM_LITERAL);

		diagram.setName(generateDiagramName(aPackage));

		helper.createViews(diagram, aPackage.getOwnedMembers());

		helper.layout(diagram, ILayoutNodesProvider.DEFAULT_LAYOUT);

		helper.openDiagramEditor(diagram);
	}

	/**
	 * Generates a new diagram name
	 * 
	 * @param aPackage
	 *            The package whose diagrams are used to determine uniqueness of
	 *            generated name
	 * @return A name that can be used to generate a diagram
	 */
	private String generateDiagramName(Package aPackage) {

		List diagrams = helper.getDiagrams(aPackage);
		String prefix = DiagramPlugin
			.getResourceString(KEY_DIAGRAM_NAME_PREFIX);
		String tentativeName = prefix;

		boolean match = true;

		/*
		 * Try tentative names until we don't find a matching diagram with that
		 * name
		 */
		for (int nextSeqNo = 1; match == true; nextSeqNo++) {

			/*
			 * Iterate the package's diagrams until we find an unused tentative
			 * name
			 */
			match = false;

			for (Iterator iter = diagrams.iterator(); iter.hasNext()
				&& match == false;) {

				Diagram diagramIter = (Diagram) iter.next();

				if (diagramIter.getName().equals(tentativeName)) {
					match = true;
				}
			}

			/*
			 * Found a diagram whose name matched the tentative name, generate a
			 * new tentative name
			 */
			if (match) {
				tentativeName = prefix + String.valueOf(nextSeqNo);
			}
		}

		return tentativeName;
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow windowIn) {
		//No-op
	}
}