/*
 * Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms
 * and conditions in the IBM International Program License Agreement.
 *
 *  Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package com.ibm.xtools.modeler.ui.pde.examples.enum.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;

import com.ibm.xtools.emf.msl.ResourceSetReadOperation;
import com.ibm.xtools.modeler.UMLModeler;
import com.ibm.xtools.modeler.ui.pde.examples.enum.EnumPlugin;

/**
 * This action delegate walks the selected elements and log the visited elements.
 * 
 * @see IWorkbenchWindowActionDelegate
 */
public class EnumModel
	implements IWorkbenchWindowActionDelegate {
	
	/**
	 * Key for name of console
	 */
	private static final String KEY_CONSOLE_NAME = "consoleName"; //$NON-NLS-1$
	
	/**
	 * Path for console icon
	 */
	private static final String CONSOLE_ICON_PATH = "icons/enum.gif"; //$NON-NLS-1$
	
	/**
	 * Key for message to output when an exception occurred
	 */
	private static final String KEY_EXCEPTION = "message.exception"; //$NON-NLS-1$
	
	/**
	 * Key for message to output when the operation was cancelled
	 */
	private static final String KEY_INTERRUPTED = "message.interrupted"; //$NON-NLS-1$
	
	/**
	 * Key for message to output when the operation cannot be performed on selection
	 */
	private static final String KEY_NO_SELECTION = "message.noSelection"; //$NON-NLS-1$
	
	/**
	 * Console stream
	 */
	private MessageConsoleStream consoleStream;

	/**
	 * Constructor
	 */
	public EnumModel() {
		//No-op
	}

	/**
	 * Walk the selected objects and log them to the console
	 * 
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {

		// Create and show the console
		showConsole();

		// Get selection
		final List elements = UMLModeler.getUMLUIHelper().getSelectedElements();
		
		if (elements.size() == 0) {
			consoleStream.println(EnumPlugin.getResourceString(KEY_NO_SELECTION));
		}

		try {
			UMLModeler.getEditingDomain().run( new ResourceSetReadOperation() {

				protected void execute(IProgressMonitor monitor)
					throws InvocationTargetException, InterruptedException {
					
					// Log each elements
					for (Iterator iter = elements.iterator(); iter.hasNext();) {
						logObject(iter.next(), ""); //$NON-NLS-1$
					}

				}
			},new NullProgressMonitor());
			
		} catch (InvocationTargetException e) {
			consoleStream.println(EnumPlugin.getResourceString(KEY_EXCEPTION)); //$NON-NLS-1$
			throw new RuntimeException(e.getCause());
		} catch (InterruptedException e) {
			consoleStream.println(EnumPlugin.getResourceString(KEY_INTERRUPTED)); //$NON-NLS-1$
		}

	}

	/**
	 * Creates and display the console used to output the owned elements' .toString()
	 */
	private void showConsole() {
		ImageDescriptor imageDesc = EnumPlugin.imageDescriptorFromPlugin(EnumPlugin.getDefault().getBundle().getSymbolicName(), CONSOLE_ICON_PATH);
		MessageConsole console = new MessageConsole(EnumPlugin.getResourceString(KEY_CONSOLE_NAME), imageDesc);
		IConsoleManager conMgr = ConsolePlugin.getDefault().getConsoleManager();
		conMgr.addConsoles(new IConsole[] {console});
		conMgr.showConsoleView(console);
		consoleStream = console.newMessageStream();
	}

	/**
	 * Logs the specified object's toString() and its content
	 * @param object The object to log 
	 * @param indent The string to prefix to the object's toString()
	 * @param consoleStream The stream to use for console output 
	 */
	private void logObject(Object object, String indent) {

		// Log object
		if (null != object) {
			consoleStream.println(indent + object.toString());
		} else {
			consoleStream.println(indent + "<null>"); //$NON-NLS-1$
		}
		
		// Log contents
		if (object instanceof EObject) {
			Iterator contents = ((EObject)object).eContents().iterator();
			while (contents.hasNext()) {
				logObject(contents.next(), indent + "\t"); //$NON-NLS-1$
			}
		}
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow windowIn) {
		//No-op
	}
}