/*
 * Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms
 * and conditions in the IBM International Program License Agreement.
 *
 *  Copyright IBM Corporation 2003, 2005. All Rights Reserved. 
 */
package com.ibm.xtools.modeler.ui.pde.examples.fan.actions;

import java.lang.reflect.InvocationTargetException;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;
import org.eclipse.uml2.ConnectableElement;
import org.eclipse.uml2.EventOccurrence;
import org.eclipse.uml2.Feature;
import org.eclipse.uml2.Lifeline;
import org.eclipse.uml2.Message;
import org.eclipse.uml2.MessageEnd;
import org.eclipse.uml2.Property;
import org.eclipse.uml2.Type;
import org.eclipse.uml2.TypedElement;
import org.eclipse.uml2.UML2Package;

import com.ibm.xtools.emf.msl.ResourceSetReadOperation;
import com.ibm.xtools.modeler.UMLModeler;
import com.ibm.xtools.modeler.ui.pde.examples.fan.FanPlugin;
import com.ibm.xtools.notation.View;

/**
 * This action delegate walks the selected elements and logs the fan-in metric of each visited
 * class elements.
 * 
 * @see IWorkbenchWindowActionDelegate
 */
public class FanAction
	implements IWorkbenchWindowActionDelegate {

	/**
	 * Key for name of console
	 */
	private static final String KEY_CONSOLE_NAME = "consoleName"; //$NON-NLS-1$

	/**
	 * Key for message to output for inappropriate selection
	 */
	private static final String KEY_INVALID_SELECTION = "message.invalidSelection"; //$NON-NLS-1$

	/**
	 * Key for message to output when inappropriate selection
	 */
	private static final String KEY_FAN_IN = "message.fanIn"; //$NON-NLS-1$

	/**
	 * Key for message to output when an exception occurred
	 */
	private static final String KEY_EXCEPTION = "message.exception"; //$NON-NLS-1$

	/**
	 * Key for message to output when the operation was cancelled
	 */
	private static final String KEY_INTERRUPTED = "message.interrupted"; //$NON-NLS-1$

	/**
	 * Path for console icon
	 */
	private static final String CONSOLE_ICON_PATH = "icons/fan.gif"; //$NON-NLS-1$

	/**
	 * Console stream
	 */
	private MessageConsoleStream consoleStream;

	/**
	 * Constructor
	 */
	public FanAction() {
		//No-op
	}

	/**
	 * Walk the selected objects and log each class' fan-in metric to the
	 * console
	 * 
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {

		// Create and show the console
		showConsole();
	
		try {
			// Model access must be wrapped within a ResourceSetReadOperation			
			UMLModeler.getEditingDomain().run( new ResourceSetReadOperation() {

				protected void execute(IProgressMonitor monitor)
					throws InvocationTargetException, InterruptedException {

					// Get selection
					List elements = UMLModeler.getUMLUIHelper().getSelectedElements();
					// keep track if we log anything					
					boolean logged = false;

					for (Iterator iter = elements.iterator(); iter.hasNext();) {
						Object object = iter.next();
						
						// If a view, try to get semantic bridge
						if (object instanceof View) {
							object = ((View)object).getElement();
						}
		
						// If a class
						if (object instanceof org.eclipse.uml2.Class) {
							org.eclipse.uml2.Class clazz = (org.eclipse.uml2.Class) object;
		
							// Calculate fan-in
							int fanIn = calcFanIn(clazz);
		
							// Log it
							String message = MessageFormat.format(FanPlugin
								.getResourceString(KEY_FAN_IN), new Object[] {
								clazz.getName(), String.valueOf(fanIn)});
		
							logged = true;
		
							consoleStream.println(message);
						}
					}
					if (false == logged) {
						String message = FanPlugin.getResourceString(KEY_INVALID_SELECTION);
						consoleStream.println(message);
					}					
				}
			}, new NullProgressMonitor());
			
		} catch (InterruptedException e) {
			String message = FanPlugin.getResourceString(KEY_INTERRUPTED);
			consoleStream.println(message);

		} catch (InvocationTargetException e1) {
			String message = FanPlugin.getResourceString(KEY_EXCEPTION);
			consoleStream.println(message);

			throw new RuntimeException(e1);
		}
	}
	
	/**
	 * Creates and display the console used to output the fan-in result
	 */
	private void showConsole() {
		ImageDescriptor imageDesc = FanPlugin.imageDescriptorFromPlugin(
			FanPlugin.getDefault().getBundle().getSymbolicName(),
			CONSOLE_ICON_PATH);
		MessageConsole console = new MessageConsole(FanPlugin
			.getResourceString(KEY_CONSOLE_NAME), imageDesc);
		IConsoleManager conMgr = ConsolePlugin.getDefault().getConsoleManager();
		conMgr.addConsoles(new IConsole[] {console});
		conMgr.showConsoleView(console);
		consoleStream = console.newMessageStream();
	}

	/**
	 * Calculates the fan-in metric of the selected element
	 * 
	 * @param clazz
	 *            The class to calculate the fan-in for
	 * @return The fan-in metric
	 * @throws InterruptedException
	 *             The operation was cancelled
	 * @throws InvocationTargetException
	 *             An exception occured that prevented fan-in calculation
	 *  
	 */
	private int calcFanIn(final org.eclipse.uml2.Class clazz) {
		// Set that holds the classes that reference the specified class
		HashSet referencingClasses = new HashSet();

		// Handle dependencies that can be inferred from operations
		getRefencingClassesByOperation(clazz, referencingClasses);

		// Handle dependencies that can be inferred from properties
		getRefencingClassesByProperty(clazz, referencingClasses);

		// Make sure not to count the specified class
		referencingClasses.remove(clazz);

		// Fan-in metric is the number of classes that reference the
		// specified class
		 return referencingClasses.size();
	}

	/**
	 * Adds to the set specified by <code>referencingClasses</code> all
	 * classes that reference the class specified by <code>clazz</code>
	 * through operations
	 * 
	 * @param clazz
	 *            The class to look for references to
	 * @param referencingClasses
	 *            The set of classes that reference the specified class
	 */
	private void getRefencingClassesByOperation(org.eclipse.uml2.Class clazz,
			HashSet referencingClasses) {

		// For each of the specified class' features
		EList features = clazz.getFeatures();
		for (Iterator iter = features.iterator(); iter.hasNext();) {
			Feature feature = (Feature) iter.next();

			// Look for referencer to the feature using their Signature feature
			EReference[] eReferences = new EReference[] {UML2Package.eINSTANCE
				.getMessage_Signature()};
			Collection messages = UMLModeler.getEObjectHelper().getReferencers(
				feature, eReferences);

			// For each referencer
			for (Iterator iterRef = messages.iterator(); iterRef.hasNext();) {

				// Make sure it is a Message
				Object object = iterRef.next();
				if (object instanceof Message) {

					Message message = (Message) object;

					// Get sender message end and add the class at the other end
					// to the set
					MessageEnd sendMessageEnd = message.getSendEvent();
					addOtherClassAtMessageEnd(sendMessageEnd,
						referencingClasses);
				}
			}
		}
	}

	/**
	 * Retrieve the class representing the lifeline covered by the MessageEnd
	 * specified by <code>messageEnd</code>, and add to the Set specified by
	 * <code>set</code>
	 * 
	 * @param messageEnd
	 *            MessageEnd to get class from
	 * @param set
	 *            Set to add the class to
	 */
	private void addOtherClassAtMessageEnd(MessageEnd messageEnd, HashSet set) {

		// try to cast as an EventOccurence
		if (messageEnd instanceof EventOccurrence) {
			EventOccurrence eventOccurence = (EventOccurrence) messageEnd;

			// There should be only one lifeline
			EList lifelines = eventOccurence.getCovereds();
			if (lifelines.size() == 1) {
				Lifeline lifeline = (Lifeline) lifelines.get(0);
				ConnectableElement connectableElement = lifeline
					.getRepresents();

				// ConnectableElement should be a property
				if (connectableElement instanceof Property) {
					Property property = (Property) connectableElement;

					// Type of property determines caller class
					Type type = property.getType();
					if (type instanceof org.eclipse.uml2.Class) {
						set.add(type);
					}
				}
			}
		}
	}

	/**
	 * Adds to the set specified by <code>referencingClasses</code> all
	 * classes that reference the class specified by <code>clazz</code>
	 * through a property
	 * 
	 * @param clazz
	 *            The class to look for references to
	 * @param referencingClasses
	 *            The set of classes that reference the specified class
	 */
	private void getRefencingClassesByProperty(org.eclipse.uml2.Class clazz,
			HashSet referencingClasses) {

		// Look for referencer to the class as a type
		EReference[] eReferences = new EReference[] {UML2Package.eINSTANCE
			.getTypedElement_Type()};

		Collection typedElements = UMLModeler.getEObjectHelper()
			.getReferencers(clazz, eReferences);

		// For each typed elements referencing the specified class
		for (Iterator iter = typedElements.iterator(); iter.hasNext();) {

			TypedElement typedElement = (TypedElement) iter.next();

			// Check if it is a Property
			if (typedElement instanceof Property) {

				Property property = (Property) typedElement;

				// Add class that owns the Property to the set
				org.eclipse.uml2.Class ownerClass = property.getClass_();

				if (null != ownerClass) {
					referencingClasses.add(ownerClass);
				}
			}
		}
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow windowIn) {
		//No-op
	}
}