/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2004.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.xtools.modeler.ui.pde.examples.properties;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.swt.widgets.Composite;

import com.ibm.xtools.common.ui.services.properties.descriptors.CompositeSourcePropertyDescriptor;
import com.ibm.xtools.emf.msl.ResourceSetModifyOperation;
import com.ibm.xtools.modeler.UMLModeler;

/**
 * An example property descriptor, which describes property of a
 * <code>EModelElement</code> instance. The descriptor uses instance of
 * <code>Property</code> class as an id object for the property at hand. The
 * instances of <code>Property</code> are defined in the
 * <code>ExamplePropetiesTable</code> for quick reference.
 * 
 * The example descriptor allows to display and edit example annotation object,
 * which is attached to the model element object. By default the example
 * annotation does not exist until user attempts to modify the property. After
 * such an attempt the annotation object is created and it's details map is
 * populated with the new property value.
 * 
 * In order to demonstrate value validation, this example creates a validator
 * object which will accept only integer values as being appropriate for this
 * property.
 * @see <code>createPropertyEditor(Composite)</code>.
 */
public class ExamplePropertyDescriptor
	extends CompositeSourcePropertyDescriptor {

	private static final String OPERATION_LABEL = PropertiesExamplesPlugin
		.getResourceString("operationLabel"); //$NON-NLS-1$

	protected static final String ERROR_MSG = PropertiesExamplesPlugin
		.getResourceString("invalidInputMessage"); //$NON-NLS-1$

	private static final String EXAMPLE_ANNOTATION_URI = "EXAMPLE_ANNOTATION_URI"; //$NON-NLS-1$

	private static final Object EXAMPLE_PROPERTY_TAG = "EXAMPLE_PROPERTY_TAG"; //$NON-NLS-1$

	/**
	 * Create a new desriptor, which describes property of a
	 * <code>EModelElement</code> instance
	 * 
	 * @param object -
	 *            property owner object
	 * @param id -
	 *            property id object
	 */
	public ExamplePropertyDescriptor(EModelElement object, Property id) {
		super(object, id, id.getDisplayName());

		setDescription(id.getDescription());
		setDefaultValue(id.getDefaultValue());
	}

	/**
	 * A convinience accessor to avoid type casts. This descriptor only
	 * describes properties of <code>EModelElement</code> instances.
	 * 
	 * @return - a model elenment - target of the property
	 */
	private EModelElement getModelElement() {
		return (EModelElement) getObject();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertyDescriptor#createPropertyEditor(org.eclipse.swt.widgets.Composite)
	 */
	public CellEditor createPropertyEditor(Composite parent) {

		if (isReadOnly())
			return null;

		CellEditor editor = new ExampleCellEditor(new ICellEditorValidator() {

			public String isValid(Object value) {
				if (value instanceof String) {
					try {
						new Integer((String) value); // make sure that the
						// string value can be
						// evaluated to an integer
						return null;
					} catch (NumberFormatException e) {
						return ERROR_MSG;
					}
				}
				return ERROR_MSG;
			}
		});

		editor.create(parent);
		return editor;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.xtools.common.ui.services.properties.descriptors.ICompositeSourcePropertyDescriptor#getPropertyValue()
	 */
	public Object getPropertyValue() {

		EAnnotation ann = getModelElement().getEAnnotation(
			EXAMPLE_ANNOTATION_URI);

		return (null == ann) ? getDefaultValue()
			: (String) ann.getDetails().get(EXAMPLE_PROPERTY_TAG);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.xtools.common.ui.services.properties.descriptors.CompositeSourcePropertyDescriptor#setValue(java.lang.Object)
	 */
	public void setValue(final Object propertyValue) {

		try {
			UMLModeler.getEditingDomain().run(
				new ResourceSetModifyOperation(PropertiesExamplesPlugin
					.getResourceString(OPERATION_LABEL)) {

					protected void execute(IProgressMonitor monitor)
						throws InvocationTargetException, InterruptedException {

						EAnnotation ann = getModelElement().getEAnnotation(
							EXAMPLE_ANNOTATION_URI);

						if (null == ann) {// create annotation object if there isn't one already
							ann = EcoreFactory.eINSTANCE.createEAnnotation();
							ann.setSource(EXAMPLE_ANNOTATION_URI);
							getModelElement().getEAnnotations().add(ann);
						}

						// save value in EMap
						ann.getDetails().put(EXAMPLE_PROPERTY_TAG,
							propertyValue);
					}

				}, new NullProgressMonitor());

		} catch (InvocationTargetException e) {
			throw new RuntimeException(e.getCause());
		} catch (InterruptedException e) {
			// Interrupted by user
		}

	}

}