/*
 * Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms
 * and conditions in the IBM International Program License Agreement.
 *
 *  Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package com.ibm.xtools.modeler.ui.pde.examples.query.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.uml2.Element;

import com.ibm.xtools.common.core.util.IAdaptableSelection;
import com.ibm.xtools.emf.msl.ResourceSetReadOperation;
import com.ibm.xtools.emf.query.ocl.InvalidOclExpressionException;
import com.ibm.xtools.modeler.UMLModeler;
import com.ibm.xtools.modeler.ui.pde.examples.query.QueryPlugin;

/**
 * This action delegate query user for the name of a class, use query APIs to
 * find the class, and select it.
 * 
 * @see IWorkbenchWindowActionDelegate
 */
public abstract class QueryActionBase
	implements IWorkbenchWindowActionDelegate {

	/**
	 * Error message to display when an operation was interrupted
	 */
	private static final String KEY_MESSAGE_INTERRUPTED = "message.interrupted"; //$NON-NLS-1$

	/**
	 * Error message to display when an exception occured
	 */
	private static final String KEY_MESSAGE_EXCEPTION = "message.exception"; //$NON-NLS-1$

	/**
	 * The shell this action is hosted in
	 */
	private Shell shell = null;

	/**
	 * Selected element
	 */
	private Element element = null;

	/**
	 * The InputDialog title
	 */
	private String title;

	/**
	 * The InputDialog message
	 */
	private String message;

	/**
	 * The InputDialog default value
	 */
	private String defaultValue;

	/**
	 * The message to output when query result set is empty
	 */
	private String notFoundMessage;

	/**
	 * Constructor
	 */
	public QueryActionBase(String titleIn, String messageIn,
			String defaultValueIn, String notFoundMessageIn) {
		title = titleIn;
		message = messageIn;
		defaultValue = defaultValueIn;
		notFoundMessage = notFoundMessageIn;
	}

	/**
	 * Requests user info, performs a query based on the info, and selects
	 * result
	 * 
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {

		if (null == element) {
			action.setEnabled(false);
			return;
		}

		final InputDialog id = new InputDialog(shell, title, message, defaultValue,
			null);
		id.setBlockOnOpen(true);

		if (InputDialog.OK == id.open()) {

			try {
				UMLModeler.getEditingDomain().run(
					new ResourceSetReadOperation() {

						protected void execute(IProgressMonitor monitor)
							throws InvocationTargetException,
							InterruptedException {

							Set result;
							
							try {
								result = performQuery(element, id.getValue(), monitor);
							} catch (InvalidOclExpressionException e) {
								throw new InvocationTargetException(e);
							}

							if (result.isEmpty()) {

								MessageDialog.openInformation(shell, title,
									notFoundMessage);
							} else {
								UMLModeler
									.getUMLUIHelper()
									.setSelectedElements(
										"com.ibm.xtools.modeler.ui.views.modelexplorer.ModelExplorer", new ArrayList(result)); //$NON-NLS-1$
							}

						}
					}, new NullProgressMonitor());
			} catch (Exception e) {
				// Exceptions are not expected
				MessageDialog.openInformation(shell, title, QueryPlugin
					.getResourceString(KEY_MESSAGE_EXCEPTION));
				throw new RuntimeException(e);
			}
		}
	}

	/**
	 * Perform a query that returns a set of objects.
	 * 
	 * @param context
	 *            The element to use as the contet for the query
	 * @param value
	 *            The value used to drive the query
	 * @param monitor
	 *            The progress monitor to use while performing the query
	 * @return The set of objects returned by the query
	 */
	protected abstract Set performQuery(Element context, String value,
			IProgressMonitor monitor)
		throws InterruptedException, InvalidOclExpressionException;

	/**
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, final ISelection selection) {

		element = null;

		try {
			UMLModeler.getEditingDomain().run(new ResourceSetReadOperation() {

				protected void execute(IProgressMonitor monitor)
					throws InvocationTargetException, InterruptedException {

					if (selection instanceof IStructuredSelection) {
						IStructuredSelection structuredSelection = (IStructuredSelection) selection;

						Object firstSelection = structuredSelection
							.getFirstElement();

						if (firstSelection instanceof IAdaptableSelection) {

							// Try to adapt to Element
							element = (Element) ((IAdaptableSelection) firstSelection)
								.getAdapter(Element.class);
						}
					}
				}
			}, new NullProgressMonitor());

		} catch (InvocationTargetException e) {

			MessageDialog.openInformation(shell, QueryPlugin
				.getResourceString(title), QueryPlugin
				.getResourceString(KEY_MESSAGE_EXCEPTION));
			throw new RuntimeException(e);

		} catch (InterruptedException e) {

			MessageDialog.openInformation(shell, QueryPlugin
				.getResourceString(title), QueryPlugin
				.getResourceString(KEY_MESSAGE_INTERRUPTED));

		}

		action.setEnabled((null != element));
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		shell = window.getShell();
	}
}