/*
 *+--------------------------------------------------------------------------------------+
 *| Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and |
 *| conditions in the IBM International Program License Agreement.						 |	
 *|  Copyright IBM Corporation 2003, 2004. All Rights Reserved.						 |
 *+--------------------------------------------------------------------------------------+
 */

package com.ibm.xtools.transform.samples.modeltotext;

import java.text.CharacterIterator;
import java.text.StringCharacterIterator;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * An utility class to generate proper Java names for identifiers from speficied
 * strings.
 */
public class JavaNameTool {

	/**
	 * An interface to specify how the specified character should be handled:
	 * convet it to lower case or upper case or leave it as is.
	 */
	public interface CharHandler {

		public char handle(char c);
	}

	/**
	 * Converts a source string into a proper Java identifier.
	 * 
	 * @param src
	 *            Source string that needs to be converted to proper Java
	 *            identifier
	 * @param firstCharHandler
	 *            Specifies how the first character should handled.
	 * @return
	 */
	private static String asIdentifier(String src, CharHandler firstCharHandler) {
		StringCharacterIterator i = new StringCharacterIterator(src);
		StringBuffer javaIdentifer = new StringBuffer(src.length());
		boolean first = true;
		for (char c = i.first(); c != CharacterIterator.DONE; c = i.next()) {
			if (first && Character.isJavaIdentifierStart(c)) {
				javaIdentifer.append(firstCharHandler.handle(c));
				first = false;
			} else if (!first && Character.isJavaIdentifierPart(c)) {
				javaIdentifer.append(c);
			}
		}
		return javaIdentifer.toString();
	}

	/**
	 * Converts a source string into a proper Java class name.
	 * 
	 * @param src
	 *            Source string
	 * @return Converted class name.
	 */
	public static String asClass(String src) {
		String identifier = asIdentifier(src, new CharHandler() {

			public char handle(char c) {
				return Character.toUpperCase(c);
			}
		});
		return identifier;
	}

	/**
	 * Converts the source string into a proper Java package name.
	 * 
	 * @param src
	 *            Source string
	 * @return The package name
	 */
	public static String asPackage(String src) {
		String identifier = asIdentifier(src, new CharHandler() {

			public char handle(char c) {
				return c;
			}
		});
		return identifier.toLowerCase();
	}

	/**
	 * Converts a source string into a proper Java attribute (or field) name.
	 * 
	 * @param src Source string
	 * @return The converted attribute name
	 */
	public static String asAttribute(String src) {
		String identifier = asIdentifier(src, new CharHandler() {

			public char handle(char c) {
				return Character.toLowerCase(c);
			}
		});
		return identifier;
	}

	/**
	 * Splits a source string into string tokens separated by the separator.
	 * 
	 * @param source Source string
	 * @param sep The separator
	 * @return Array of string tokens
	 */
	public static String[] split(String source, String sep) {
		if (source == null) {
			return new String[] {};
		}
		if (sep == null) {
			return new String[] {source};
		}

		List strings = new ArrayList();
		for (StringTokenizer st = new StringTokenizer(source, sep); st
			.hasMoreTokens();) {
			strings.add(st.nextToken().trim());
		}

		return (String[]) strings.toArray(new String[strings.size()]);
	}

}