/*
 *+--------------------------------------------------------------------------------------+
 *| Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and |
 *| conditions in the IBM International Program License Agreement.						 |	
 *|  Copyright IBM Corporation 2003, 2004. All Rights Reserved.						 |
 *+--------------------------------------------------------------------------------------+
 */

package com.ibm.xtools.transform.samples.modeltotext;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.ibm.xtools.transform.core.AbstractTransform;
import com.ibm.xtools.transform.core.AbstractTransformationProvider;
import com.ibm.xtools.transform.core.ITransformContext;
import com.ibm.xtools.transform.core.ITransformationDescriptor;
import com.ibm.xtools.transform.samples.modeltotext.classtotext.console.transforms.ClassToTextConsoleRootTransform;
import com.ibm.xtools.transform.samples.modeltotext.classtotext.file.transforms.ClassToTextFileRootTransform;
import com.ibm.xtools.transform.samples.modeltotext.l10n.ResourceManager;
import com.ibm.xtools.transform.samples.modeltotext.statetojsp.transforms.StateToJspRootTransform;

/**
 * Provider for the sample model-to-text transformations.
 * 
 * @see AbstractTransformationProvider
 */
public class ModelToTextTransformationProvider extends
        AbstractTransformationProvider {

    /**
     * Constructor.
     */
    public ModelToTextTransformationProvider() {
        super();
    }

    /**
	 * @see com.ibm.xtools.transform.core.AbstractTransformationProvider#createTransformation(com.ibm.xtools.transform.core.ITransformationDescriptor)
	 */
    public AbstractTransform createTransformation(ITransformationDescriptor descriptor) {
        AbstractTransform transform = null;
        if (ClassToTextConsoleRootTransform.ID.equals(descriptor.getId())) {
            transform = new ClassToTextConsoleRootTransform(descriptor);
        } else if (ClassToTextFileRootTransform.ID.equals(descriptor.getId())) {
            transform = new ClassToTextFileRootTransform(descriptor);
        } else if (StateToJspRootTransform.ID.equals(descriptor.getId())) {
            transform = new StateToJspRootTransform(descriptor);
        }
        return transform;
    }
    

    /**
     * Validates that the source and the target container specified in the
     * context are valid for this transformation.
     * 
     * @see com.ibm.xtools.transform.core.internal.services.ITransformationProvider#validateContext(com.ibm.xtools.transform.core.internal.services.ITransformationDescriptor,
     *      com.ibm.xtools.transform.core.internal.engine.ITransformContext)
     */
    public IStatus validateContext(ITransformationDescriptor descriptor,
            ITransformContext context) {
    	if (ClassToTextConsoleRootTransform.ID.equals(descriptor.getId())) {
            return validateContextForClassToTextConsoleTransform(descriptor, context);
        } else if (ClassToTextFileRootTransform.ID.equals(descriptor.getId())) {
            return validateContextForClassToTextFileTransform(descriptor, context);
        } else if (StateToJspRootTransform.ID.equals(descriptor.getId())) {
            return validateContextForStateToJspTransform(descriptor, context);
        } else {
        	return super.validateContext(descriptor, context);
        }
    }
    
    /**
     * Validates that the source and the target container specified in the
     * context are valid for the Class to Text -> Console transformation.
     * 
     * @see com.ibm.xtools.transform.core.internal.services.ITransformationProvider#validateContext(com.ibm.xtools.transform.core.internal.services.ITransformationDescriptor,
     *      com.ibm.xtools.transform.core.internal.engine.ITransformContext)
     */
    private IStatus validateContextForClassToTextConsoleTransform(ITransformationDescriptor descriptor,
            ITransformContext context) {
        AbstractTransform transform = context.getTransform();
        
        boolean isSourceOk = transform.canAccept(context);
		
		// We don't care about the target
		
		String pluginId = ModelToTextPlugin.getPluginId();
		
		if (isSourceOk) {
			String okMsg = ResourceManager.getString("ModelToTextTransformationProvider.context.validMsg"); //$NON-NLS-1$
			return new Status(IStatus.OK, pluginId,
                AbstractTransformationProvider.CODE_SOURCE,
				okMsg,
                null);
		} else {
			String errorMsg = ResourceManager.getString("ClassToTextConsoleTransformation.invalidSourceMsg"); //$NON-NLS-1$
			return new Status(IStatus.ERROR, pluginId,
	            AbstractTransformationProvider.CODE_SOURCE, errorMsg, null);
		}
    }
    
    /**
     * Validates that the source and the target container specified in the
     * context are valid for the Class to Text -> File transformation.
     * 
     * @see com.ibm.xtools.transform.core.internal.services.ITransformationProvider#validateContext(com.ibm.xtools.transform.core.internal.services.ITransformationDescriptor,
     *      com.ibm.xtools.transform.core.internal.engine.ITransformContext)
     */
    private IStatus validateContextForClassToTextFileTransform(ITransformationDescriptor descriptor,
			ITransformContext context) {

    	AbstractTransform transform = context.getTransform();
    	
		boolean isSourceOk = transform.canAccept(context);
		
		Object target = context.getTargetContainer();
		boolean isTargetOk = (target instanceof IProject) ? true : false;
		boolean arePropertiesOk = true;

		final String sourceErrMsg = ResourceManager.getString("ClassToTextFileTransformation.invalidSourceMsg"); //$NON-NLS-1$
		final String targetErrMsg = ResourceManager.getString("ClassToTextFileTransformation.invalidTargetMsg"); //$NON-NLS-1$
		final String pluginId = ModelToTextPlugin.getPluginId();
		
		return StatusUtility.createTransformContextValidationStatus(
			pluginId, 
			isSourceOk, sourceErrMsg, 
			isTargetOk, targetErrMsg,
			arePropertiesOk);
	}
    

    /**
     * Validates that the source and the target container specified in the
     * context are valid for the State to JSP transformation.
     * 
     * @see com.ibm.xtools.transform.core.internal.services.ITransformationProvider#validateContext(com.ibm.xtools.transform.core.internal.services.ITransformationDescriptor,
     *      com.ibm.xtools.transform.core.internal.engine.ITransformContext)
     */
    private IStatus validateContextForStateToJspTransform(ITransformationDescriptor descriptor,
			ITransformContext context) {

    	AbstractTransform transform = context.getTransform();
		boolean sourceOk = transform.canAccept(context);
		Object target = context.getTargetContainer();
		boolean targetOk = (target instanceof IProject) ? true : false;
		boolean propertiesOk = true;

		final String sourceErrMsg = ResourceManager.getString("StateToJspTransformation.invalidSourceMsg"); //$NON-NLS-1$
		final String targetErrMsg = ResourceManager.getString("StateToJspTransformation.invalidTargetMsg"); //$NON-NLS-1$
		final String pluginId = ModelToTextPlugin.getPluginId();
		
		return StatusUtility.createTransformContextValidationStatus(
			pluginId, 
			sourceOk, sourceErrMsg, 
			targetOk, targetErrMsg,
			propertiesOk);
	}
}
