/*
 *+--------------------------------------------------------------------------------------+
 *| Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and |
 *| conditions in the IBM International Program License Agreement.						 |	
 *|  Copyright IBM Corporation 2003, 2004. All Rights Reserved.						 |
 *+--------------------------------------------------------------------------------------+
 */

package com.ibm.xtools.transform.samples.modeltotext;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.Writer;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;

/**
 * An utility class to contain methods to manipulate Eclipse resources.
 */
public class ResourceUtility {
	
	private static String ENCODING_UTF8 = "UTF-8"; //$NON-NLS-1$

	/**
	 * Constructor. Being a utility class with all static methods and no state
	 * of its own, this class does not need its constaructor to be public.
	 */
	public ResourceUtility() {
		super();
	}

	/**
	 * Returns the handle to a file in a folder inside a project. The file or
	 * the folder may not yet exist.
	 * 
	 * Given a project this method creates and returns a handle to a identified
	 * file. The folder or the file may not yet exist. If the identified folder
	 * does not exist, the method creates the folder under the project, and then
	 * creates a file handle to the file.
	 * 
	 * @param project
	 *            Handle to the project containing the folder,
	 *            <code>folderName</code> and the file <code>fileName</code>
	 * @param folderName
	 *            The name of the folder containing the file
	 *            <code>filename</code>
	 * @param fileName
	 *            The name of the file
	 * @return A handle to the file. If the file does not exist in the folder
	 *         returnvalue.exists() will false.
	 * @throws CoreException
	 *             If the creation of the folder (in the case it does not exist)
	 *             fails
	 */
	public static IFile getFileFromProjectFolder(IProject project,
			String folderName, String fileName)
		throws CoreException {
		IFolder folder = project.getFolder(folderName);
		if (!folder.exists()) {
			folder.create(true, true, new NullProgressMonitor());
		}
		IFile file = ResourcesPlugin
			.getWorkspace()
			.getRoot()
			.getFileForLocation(new Path(folder.getLocation() + "/" + fileName));//$NON-NLS-1$
		return file;
	}

	/**
	 * Writes the contents of a Writer to a specified file in a specified folder
	 * in a specified project. If the file exits, it overwrites the contents
	 * only if the flag, overwrite, is true.
	 * 
	 * @param writer
	 *            Source of the file content
	 * @param project
	 *            Handle to the project containing the folder,
	 *            <code>folderName</code> and the file <code>fileName</code>
	 * @param folderName
	 *            The name of the folder containing the file
	 *            <code>filename</code>
	 * @param fileName
	 *            The name of the file
	 * @param overwrite
	 *            Flag to indicate if an existing file should be over-written.
	 * @throws CoreException
	 */
	public static void writeToProjectFile(Writer writer, IProject project,
			String folderName, String fileName, boolean overwrite)
		throws CoreException {
		InputStream istream = new ByteArrayInputStream(writer.toString()
			.getBytes());

		IFile file = getFileFromProjectFolder(project, folderName, fileName);
		NullProgressMonitor progressMonitor = new NullProgressMonitor();
		if (!file.exists()) {
			file.create(istream, true, progressMonitor);
			file.setCharset(ENCODING_UTF8, progressMonitor);
		} else if (overwrite == true) {
			file.setContents(istream, true, true, progressMonitor);
		}
	}

	/**
	 * Gets the project a model element is part of.
	 * 
	 * @param element
	 *            The model element. Checks assertion that <code>element</code>
	 *            is not null
	 * @return A handle to the project containing the model element
	 */
	public static IProject getProject(EObject element) {
		URI uri = element.eResource().getURI();
		IFile modelFile = ResourcesPlugin.getWorkspace().getRoot()
			.getFileForLocation(new Path(uri.device() + uri.path()));

		return modelFile.getProject();
	}

}